@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* kdc/fast_util.c */
 /*
- * kdc/fast_util.c
- *
  * Copyright (C) 2009 by the Massachusetts Institute of Technology.
  * All rights reserved.
  *
@@ -60,7 +59,8 @@ static krb5_error_code armor_ap_request
     if (retval != 0) {
         const char * errmsg = krb5_get_error_message(kdc_context, retval);
         krb5_set_error_message(kdc_context, retval,
-                               "%s while handling ap-request armor", errmsg);
+                               _("%s while handling ap-request armor"),
+                               errmsg);
         krb5_free_error_message(kdc_context, errmsg);
     }
     if (retval == 0) {
@@ -68,7 +68,8 @@ static krb5_error_code armor_ap_request
                                               tgs_server,
                                               ticket->server)) {
             krb5_set_error_message(kdc_context, KRB5KDC_ERR_SERVER_NOMATCH,
-                                   "ap-request armor for something other than  the local TGS");
+                                   _("ap-request armor for something other "
+                                     "than the local TGS"));
             retval = KRB5KDC_ERR_SERVER_NOMATCH;
         }
     }
@@ -76,7 +77,7 @@ static krb5_error_code armor_ap_request
         retval = krb5_auth_con_getrecvsubkey(kdc_context, authcontext, &subkey);
         if (retval != 0 || subkey == NULL) {
             krb5_set_error_message(kdc_context, KRB5KDC_ERR_POLICY,
-                                   "ap-request armor without subkey");
+                                   _("ap-request armor without subkey"));
             retval = KRB5KDC_ERR_POLICY;
         }
     }
@@ -125,11 +126,12 @@ kdc_find_fast(krb5_kdc_req **requestptr,
               krb5_data *checksummed_data,
               krb5_keyblock *tgs_subkey,
               krb5_keyblock *tgs_session,
-              struct kdc_request_state *state)
+              struct kdc_request_state *state,
+              krb5_data **inner_body_out)
 {
     krb5_error_code retval = 0;
     krb5_pa_data *fast_padata, *cookie_padata = NULL;
-    krb5_data scratch;
+    krb5_data scratch, *inner_body = NULL;
     krb5_fast_req * fast_req = NULL;
     krb5_kdc_req *request = *requestptr;
     krb5_fast_armored_req *fast_armored_req = NULL;
@@ -137,6 +139,8 @@ kdc_find_fast(krb5_kdc_req **requestptr,
     krb5_boolean cksum_valid;
     krb5_keyblock empty_keyblock;
 
+    if (inner_body_out != NULL)
+        *inner_body_out = NULL;
     scratch.data = NULL;
     krb5_clear_error_message(kdc_context);
     memset(&empty_keyblock, 0, sizeof(krb5_keyblock));
@@ -150,16 +154,17 @@ kdc_find_fast(krb5_kdc_req **requestptr,
             switch (fast_armored_req->armor->armor_type) {
             case KRB5_FAST_ARMOR_AP_REQUEST:
                 if (tgs_subkey) {
-                    krb5_set_error_message( kdc_context, KRB5KDC_ERR_PREAUTH_FAILED,
-                                            "Ap-request armor not permitted with TGS");
-                    retval =  KRB5KDC_ERR_PREAUTH_FAILED;
+                    retval = KRB5KDC_ERR_PREAUTH_FAILED;
+                    krb5_set_error_message(kdc_context, retval,
+                                           _("Ap-request armor not permitted "
+                                             "with TGS"));
                     break;
                 }
                 retval = armor_ap_request(state, fast_armored_req->armor);
                 break;
             default:
                 krb5_set_error_message(kdc_context, KRB5KDC_ERR_PREAUTH_FAILED,
-                                       "Unknow FAST armor type %d",
+                                       _("Unknown FAST armor type %d"),
                                        fast_armored_req->armor->armor_type);
                 retval = KRB5KDC_ERR_PREAUTH_FAILED;
             }
@@ -171,9 +176,10 @@ kdc_find_fast(krb5_kdc_req **requestptr,
                                               tgs_session, "ticketarmor",
                                               &state->armor_key);
             else {
-                krb5_set_error_message(kdc_context, KRB5KDC_ERR_PREAUTH_FAILED,
-                                       "No armor key but FAST armored request present");
                 retval = KRB5KDC_ERR_PREAUTH_FAILED;
+                krb5_set_error_message(kdc_context, retval,
+                                       _("No armor key but FAST armored "
+                                         "request present"));
             }
         }
         if (retval == 0) {
@@ -189,6 +195,14 @@ kdc_find_fast(krb5_kdc_req **requestptr,
                                     &plaintext);
             if (retval == 0)
                 retval = decode_krb5_fast_req(&plaintext, &fast_req);
+            if (retval == 0 && inner_body_out != NULL) {
+                retval = fetch_asn1_field((unsigned char *)plaintext.data,
+                                          1, 2, &scratch);
+                if (retval == 0) {
+                    retval = krb5_copy_data(kdc_context, &scratch,
+                                            &inner_body);
+                }
+            }
             if (plaintext.data)
                 free(plaintext.data);
         }
@@ -200,14 +214,15 @@ kdc_find_fast(krb5_kdc_req **requestptr,
                                             &cksum_valid);
         if (retval == 0 && !cksum_valid) {
             retval = KRB5KRB_AP_ERR_MODIFIED;
-            krb5_set_error_message(kdc_context, KRB5KRB_AP_ERR_MODIFIED,
-                                   "FAST req_checksum invalid; request modified");
+            krb5_set_error_message(kdc_context, retval,
+                                   _("FAST req_checksum invalid; request "
+                                     "modified"));
         }
         if (retval == 0) {
             if (!krb5_c_is_keyed_cksum(cksum->checksum_type)) {
                 retval = KRB5KDC_ERR_POLICY;
-                krb5_set_error_message(kdc_context, KRB5KDC_ERR_POLICY,
-                                       "Unkeyed checksum used in fast_req");
+                krb5_set_error_message(kdc_context, retval,
+                                       _("Unkeyed checksum used in fast_req"));
             }
         }
         if (retval == 0) {
@@ -219,8 +234,7 @@ kdc_find_fast(krb5_kdc_req **requestptr,
                                          KRB5_PADATA_FX_COOKIE);
         if (retval == 0) {
             state->fast_options = fast_req->fast_options;
-            if (request->kdc_state == state)
-                request->kdc_state = NULL;
+            fast_req->req_body->msg_type = request->msg_type;
             krb5_free_kdc_req( kdc_context, request);
             *requestptr = fast_req->req_body;
             fast_req->req_body = NULL;
@@ -245,6 +259,11 @@ kdc_find_fast(krb5_kdc_req **requestptr,
             }
         }
     }
+    if (retval == 0 && inner_body_out != NULL) {
+        *inner_body_out = inner_body;
+        inner_body = NULL;
+    }
+    krb5_free_data(kdc_context, inner_body);
     if (fast_req)
         krb5_free_fast_req( kdc_context, fast_req);
     if (fast_armored_req)
